@@ -175,12 +175,17 @@
 
 /* Number of file descriptors reserved for non-incoming-clients.
  * One of which may be used by auth, the rest are really reserved.
- * They can be used for outgoing server links, listeners, logging, etc.
+ * They can be used for outgoing server links, listeners, logging,
+ * DNS lookups, HTTPS callbacks, etc.
  */
-#if MAXCONNECTIONS > 1024
- #define CLIENTS_RESERVE 8
+#if MAXCONNECTIONS >= 10000
+ #define CLIENTS_RESERVE 250
+#elif MAXCONNECTIONS >= 2048
+ #define CLIENTS_RESERVE 32
+#elif MAXCONNECTIONS >= 1024
+ #define CLIENTS_RESERVE 16
 #else
- #define CLIENTS_RESERVE 4
+ #define CLIENTS_RESERVE 8
 #endif
 
 /*
@@ -294,6 +299,17 @@
 #define UNREALIRCD_DEFAULT_ECDH_CURVES "secp521r1:secp384r1:prime256v1"
 #endif
 
+/* These can be changed via set::central-spamfilter::url and ::feed */
+#define DEFAULT_CENTRAL_SPAMFILTER_URL_OPEN_ACCESS "https://spamfilter.unrealircd.org/spamfilter/v6/$feed/central_spamfilter.conf"
+#define DEFAULT_CENTRAL_SPAMFILTER_URL_RESTRICTED_ACCESS "https://spamfilter.unrealircd-api.org/spamfilter/v6/$feed/central_spamfilter.conf"
+#define DEFAULT_CENTRAL_SPAMFILTER_FEED "standard"
+
+/* These are just defaults, which you can override via set::dns */
+#define DNS_DEFAULT_CLIENT_TIMEOUT 1500
+#define DNS_DEFAULT_CLIENT_RETRIES 2
+#define DNS_DEFAULT_DNSBL_TIMEOUT 3000
+#define DNS_DEFAULT_DNSBL_RETRIES 2
+
 /* ------------------------- END CONFIGURATION SECTION -------------------- */
 #define MOTD MPATH
 #define RULES RPATH
